#!/usr/bin/env python

"""
Handle Confluence wiki requests employing special identifiers that should map to
wiki pages.
"""

import cgi, os, sys

# Location of the mapping.

MAPPING_ID_TO_PAGE = "mapping-id-to-page.txt"

# An empty string means that the wiki is anchored at the site root.

URL_PREFIX = ""

# See the scripts/tiny.py program for similar code in a stand-alone program.

from base64 import b64decode
from struct import unpack

def identifier(s):
    if len(s) > 6:
        return None
    bytes = b64decode(s.replace("-", "/").replace("_", "+") + "=" * (6 - len(s)))
    return str(unpack("<I", bytes + "\x00" * (4 - len(bytes)))[0])

# Utility functions.

def fail(pageid):
    print """\
Status: 404 Page not found
Content-Type: text/html

<html>
<head><title>Bad Page Identifier</title></head>
<body>
<h1>Bad Page Identifier</h1>
<p>The identifier given in the URL%s does not seem to refer to a page in this wiki.</p>
</body>
</html>
""" % (pageid and " (%s)" % pageid or "")
    sys.exit(0)

def redirect(pagename):
    location = "%s/%s" % (URL_PREFIX, pagename)

    print """\
Status: 302 Redirect to page
Location: %s
Content-Type: text/html

<html>
<head><title>Redirecting to Page</title></head>
<body>
<h1>Redirecting to Page</h1>
<p>If you see this message, try following <a href="%s">this link</a>.</p>
</body>
</html>
""" % (location, cgi.escape(location, True))
    sys.exit(0)

def find(f, pageid):
    for line in f.xreadlines():
        columns = line.strip().split("\t")
        if columns[0] == pageid:
            redirect(columns[1])

def main():
    args = cgi.parse_qs(os.environ.get("QUERY_STRING", ""))
    path = os.environ.get("PATH_INFO", "").strip("/")

    pageid = args.get("pageId", [None])[0] or identifier(path)
    if pageid is None:
        fail(pageid)

    f = open(MAPPING_ID_TO_PAGE)
    try:
        # With an identifier, find the corresponding page name.

        find(f, pageid)

        # Didn't find the page.

        fail(pageid)

    finally:
        f.close()

if __name__ == "__main__":
    main()

# vim: tabstop=4 expandtab shiftwidth=4
